package com.zheng.demo.domain;

import static org.junit.Assert.assertEquals;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.temporal.TemporalAdjusters;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import org.junit.Before;
import org.junit.Test;

public class FruitTest {

	private List<Fruit> fruits;

	@Before
	public void setup() {
		LocalDate now = LocalDate.now();
		fruits = Arrays.asList(new Fruit(1, "Apple", now.with(TemporalAdjusters.previous(DayOfWeek.FRIDAY))),
				new Fruit(2, "Orange", now.with(TemporalAdjusters.previous(DayOfWeek.THURSDAY))),
				new Fruit(3, "Banana", now.with(TemporalAdjusters.previous(DayOfWeek.WEDNESDAY))),
				new Fruit(4, "Peach", now.with(TemporalAdjusters.previous(DayOfWeek.SATURDAY))));
		System.out.println("Original: " + fruits);
	}

	@Test
	public void sort_basedon_Date() {
		fruits.sort(Comparator.comparing(Fruit::getInstockDate));
		assertEquals(3, fruits.get(0).getId());
		assertEquals(2, fruits.get(1).getId());
		assertEquals(1, fruits.get(2).getId());
		assertEquals(4, fruits.get(3).getId());
		System.out.println("Sorted By Date: " + fruits);

	}

	@Test
	public void filter_basedon_Date() {
		LocalDate previousThursday = LocalDate.now().with(TemporalAdjusters.previous(DayOfWeek.THURSDAY));
		List<Fruit> olderFruits = fruits.stream().filter(f -> f.getInstockDate().isBefore(previousThursday))
				.collect(Collectors.toList());

		assertEquals(3, olderFruits.get(0).getId());
		System.out.println("Fruits is older than previous Thursday: " + olderFruits);
	}

	@Test
	public void find_basedon_date() {
		List<Fruit> previoyWednesdayFruits = fruits.stream().filter(
				f -> f.getInstockDate().isEqual(LocalDate.now().with(TemporalAdjusters.previous(DayOfWeek.WEDNESDAY))))
				.collect(Collectors.toList());
		
		assertEquals(3, previoyWednesdayFruits.get(0).getId());
		System.out.println("Fruits is at previous Wednesday: " + previoyWednesdayFruits);
	}

}
